importScripts('ExtPay.js');

var extpay = ExtPay('canvashack');
extpay.startBackground();

chrome.runtime.onMessage.addListener(function(request, sender, sendResponse) {
  if (request.action === 'openPurchasePopup') {
      chrome.action.openPopup();
      chrome.storage.local.set({ showPurchasePage: true });
  }
});

const activate = () => {
  activate.actions = activate.actions || [];

  if (activate.busy) {
    return;
  }
  activate.busy = true;

  chrome.storage.local.get({
    enabled: true,
    hosts: [],
    paid: true, // Set paid to true for everyone
    lifetimePaid: true, // Set lifetimePaid to true for everyone
    blockedUrls: []  // Get the blocked URLs from storage
  }, async prefs => {
    try {
      await chrome.scripting.unregisterContentScripts();

      if (prefs.enabled && (prefs.paid || prefs.lifetimePaid)) {
        const props = {
          'allFrames': true,
          'matchOriginAsFallback': true,
          'runAt': 'document_start',
          'matches': ['*://*/*']
        };

        // Filter out invalid URLs (no ports allowed) and map to proper format
        const blockedHosts = prefs.blockedUrls
          .map(url => {
            try {
              const cleanedUrl = new URL(url.trim());
              return `*://${cleanedUrl.hostname}/*`;  // Only the hostname part
            } catch (e) {
              console.error(`Invalid URL skipped: ${url}`);
              return null;  // Skip invalid URLs
            }
          })
          .filter(url => url !== null);  // Ensure only valid URLs are included

        if (blockedHosts.length > 0) {
          await chrome.scripting.registerContentScripts([{
            ...props,
            'id': 'main',
            'js': ['inject.js'],
            'world': 'MAIN',
            'excludeMatches': blockedHosts  // Exclude blocked URLs from injection
          }]);

          console.log("Scripts registered with URL blocking logic.");
        } else {
          // If no blocked URLs, register content scripts for all pages
          await chrome.scripting.registerContentScripts([{
            ...props,
            'id': 'main',
            'js': ['inject.js'],
            'world': 'MAIN'
          }]);

          console.log("Scripts registered for all pages (no blocked URLs).");
        }
      } else {
        console.log("Scripts not registered as user is not paid.");
      }
    } catch (e) {
      console.error('CanvasHack Blocker Registration Failed', e);
    }

    activate.actions.length = 0;
    activate.busy = false;
  });
};

// Run activate when the extension starts up or is installed
chrome.runtime.onStartup.addListener(activate);
chrome.runtime.onInstalled.addListener(activate);

// Listen to changes in storage (e.g., when the user pays)
chrome.storage.onChanged.addListener(ps => {
  if (ps.enabled || ps.paid || ps.lifetimePaid) {
    activate(); // Re-run the activate function if the payment status changes
  }
});

// Listen to messages from content or popup scripts to reload tabs or handle payments
chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
  if (request.action === "reloadTab") {
    chrome.tabs.reload(sender.tab.id);
  }
});

// External listener to handle payment confirmation from the API
chrome.runtime.onMessageExternal.addListener(function (message, sender, sendResponse) {
  if (message.type === 'payment_confirmed') {
    const isLifetime = message.lifetimePaid || false;
    const customerEmail = message.customerEmail;

    // Update storage with payment status and email
    chrome.storage.local.set({ lifetimePaid: isLifetime, paid: true, customerEmail: customerEmail }, () => {
      console.log('Payment status updated in storage.');
    });

    // Broadcast payment status to all components
    chrome.runtime.sendMessage({
      action: 'payment_status_update',
      paid: true,
      lifetimePaid: isLifetime,
      customerEmail: customerEmail
    });

    sendResponse({ status: 'success', message: 'Payment status updated in the extension.' });
  } else {
    sendResponse({ status: 'failure', message: 'Unknown message type.' });
  }
});

// ExtPay listener for payment updates
extpay.onPaid.addListener(user => {
  chrome.storage.local.set({ paid: true }, function () {
    chrome.runtime.sendMessage({ action: 'updateUI', paid: true });
  });
});

chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
  if (request.action === 'updateBlockedUrls') {
    activate(); // Re-run the activate function when blocked URLs are updated
  }
});
