const overrideVisibilityProperties = () => {
    const properties = {
        visibilityState: 'visible',
        hidden: false,
        webkitVisibilityState: 'visible',
        webkitHidden: false
    };
    Object.keys(properties).forEach(prop => {
        Object.defineProperty(document, prop, { value: properties[prop], writable: true });
    });
};

const overrideFocusState = () => {
    Document.prototype.hasFocus = () => true;
};

// List of events to block
const eventsToBlock = [
    'focus', 'blur', 'visibilitychange', 'webkitvisibilitychange', 
    'pagehide', 'pageshow'
];

// Function to override event listeners
const overrideEventListeners = (target) => {
    const originalAddEventListener = target.prototype.addEventListener;
    const originalRemoveEventListener = target.prototype.removeEventListener;

    target.prototype.addEventListener = (type, listener, options) => {
        if (eventsToBlock.includes(type)) {
            console.log(`[Canvas Hack] '${type}' event listener subscription prevented.`);
        } else {
            originalAddEventListener.call(this, type, listener, options);
        }
    };

    target.prototype.removeEventListener = (type, listener, options) => {
        if (eventsToBlock.includes(type)) {
            console.log(`[Canvas Hack] '${type}' event listener removal prevented.`);
        } else {
            originalRemoveEventListener.call(this, type, listener, options);
        }
    };
};

// Function to rewrite requests and prevent tab changes
const overrideWindowEventListener = () => {
    const realEventListener = Window.prototype.addEventListener;
    Window.prototype.addEventListener = (event, listener, options) => {
        if (['focus', 'blur', 'visibilitychange'].includes(event)) {
            console.log(`[AD] '${event}' event listener subscription prevented. (Canvas Hack)`);
        } else {
            realEventListener(event, listener, options);
        }
    };
};

// Protects against AI Trojan Horse hack
const protectAgainstAITrojanHorse = () => {
    const removeOrHideElements = () => {
        const targetElement = document.querySelector('#content-wrapper .description.user_content.enhanced[data-resource-type="assignment.body"]');
        if (targetElement) {
            const spans = targetElement.querySelectorAll('span[aria-hidden="true"]');
            spans.forEach(span => {
                try {
                    span.remove();
                } catch (e) {
                    span.style.display = 'none';
                }
            });
            console.log("AI Trojan Horse Hack Protected & Prevented");
        } else {
            console.log("Target element not found");
        }
    };

    const initObserver = () => {
        const observer = new MutationObserver(removeOrHideElements);
        observer.observe(document.body, { childList: true, subtree: true });
    };

    const checkMetaAndInitialize = () => {
        const metaElement = document.querySelector('meta[name="apple-itunes-app"][content="app-id=480883488"]');
        if (metaElement) {
            console.log("Meta tag found, initializing observer and removing elements");
            initObserver();
            removeOrHideElements();
        } else {
            console.log("Meta tag not found, script will not run");
        }
    };

    if (document.readyState === 'loading') {
        document.addEventListener('DOMContentLoaded', checkMetaAndInitialize);
    } else {
        checkMetaAndInitialize();
    }
};

// Execute all functions
const initialize = () => {
    overrideVisibilityProperties();
    overrideFocusState();
    overrideEventListeners(Window);
    overrideEventListeners(Document);
    overrideWindowEventListener();
    protectAgainstAITrojanHorse();
};

initialize();
