document.addEventListener('DOMContentLoaded', function () {
    const extpay = ExtPay('canvashack');

    const apiURL = 'https://v-games-5-canvas-hack.studentmath.site/'; // https://v-games-5-canvas-hack.studentmath.site/ or https://v-games-5-canvas-hack.studentmath.site/

    // Start background process for ExtPay
    extpay.startBackground();

    // Forcefully set the user as paid for lifetime (override payment check)
    chrome.storage.local.set({ paid: true, lifetimePaid: true }, function() {
        console.log("User is marked as lifetime paid.");
        updateUI(true);  // Update the UI to reflect lifetime paid status
    });

    document.getElementById("manageButton").onclick = () => extpay.openPaymentPage();
    document.getElementById("loginButton").onclick = () => extpay.openLoginPage();

    chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
        if (message.action === 'payment_status_update') {
            console.log('Received payment status update:', message.paid);
            chrome.storage.local.set({ paid: message.paid, lifetimePaid: message.lifetimePaid }, function() {
                updateUI(message.paid || message.lifetimePaid);
            });
        }
    });

    function showOnlyPage(pageToShow) {
        const pages = [pageOne, pageTwo, pageThree, pageFour, pageFive, pageSix, pageSeven];
        pages.forEach(page => page.style.display = 'none');
        pageToShow.style.display = 'flex';
    
        if (pageToShow !== pageTwo) {
            settingsIcon.classList.remove('activated');
        }
        if (pageToShow !== pageSeven) {
            updatesIcon.classList.remove('activated');
        }
    }    
    
    const monthlyOption = document.getElementById('monthlyOption');
    const lifetimeOption = document.getElementById('lifetimeOption');
    const purchaseButton = document.querySelector('.purchase-button');

    // Function to handle switching between active/inactive pricing options
    function activateOption(selectedOption) {
        if (selectedOption === 'monthly') {
            // Set monthly as active
            monthlyOption.classList.remove('unactive');
            monthlyOption.classList.add('active');
            monthlyOption.querySelector('i').className = 'fa-solid fa-circle-check';
            
            // Set lifetime as inactive
            lifetimeOption.classList.remove('active');
            lifetimeOption.classList.add('unactive');
            lifetimeOption.querySelector('i').className = 'fa-regular fa-circle';
            
            // Update purchase button ID for monthly
            purchaseButton.id = 'paymentButton';
            attachPurchaseHandler('monthly');
        } else if (selectedOption === 'lifetime') {
            // Set lifetime as active
            lifetimeOption.classList.remove('unactive');
            lifetimeOption.classList.add('active');
            lifetimeOption.querySelector('i').className = 'fa-solid fa-circle-check';
            
            // Set monthly as inactive
            monthlyOption.classList.remove('active');
            monthlyOption.classList.add('unactive');
            monthlyOption.querySelector('i').className = 'fa-regular fa-circle';
            
            // Update purchase button ID for lifetime
            purchaseButton.id = 'paymentButtonLifetime';
            attachPurchaseHandler('lifetime');
        }
    }

    // Attach event listeners for purchase button
    function attachPurchaseHandler(option) {
        const purchaseButton = document.querySelector('.purchase-button');
        
        if (option === 'monthly') {
            purchaseButton.onclick = () => extpay.openPaymentPage(); // Monthly payment logic
            document.getElementsByClassName('purchase-button')[0].addEventListener('click', function() {
                document.getElementById('p3').style.display = 'none';
                document.getElementById('p5').style.display = 'flex';
            });
        } else if (option === 'lifetime') {
            purchaseButton.onclick = async () => {
                try {
                    console.log("Creating Stripe session...");
                    const response = await fetch(`${apiURL}/create-checkout-session`, {
                        method: 'POST',
                        headers: { 'Content-Type': 'application/json' },
                    });

                    if (response.ok) {
                        const { url, session_id } = await response.json();
                        console.log(`Stripe session created: ${session_id}`);
                        window.open(url, 'Purchase', 'width=600,height=800,focused:true,left=450');
                    } else {
                        alert('Failed to create Stripe session.');
                    }
                } catch (error) {
                    console.error('Error creating Stripe session:', error);
                }
            };
            document.getElementsByClassName('purchase-button')[0].addEventListener('click', function() {
                document.getElementById('p3').style.display = 'none';
                document.getElementById('p5').style.display = 'flex';
            });
        }
    }

    // Add event listeners to switch pricing options
    monthlyOption.addEventListener('click', function () {
        activateOption('monthly');
    });

    lifetimeOption.addEventListener('click', function () {
        activateOption('lifetime');
    });

    // Initialize with lifetime option as default
    activateOption('lifetime');

    chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
        if (message.action === "stripe_payment_success") {
            chrome.storage.local.set({ lifetimePaid: true }, () => {
                console.log("User is now a lifetime member.");
                updateUI(true);  // Update the UI to reflect paid status
            });
            chrome.storage.local.set({ paid: true }, () => {
                console.log("User is now a lifetime member.");
                updateUI(true);  // Update the UI to reflect paid status
            });
        }
    });

    chrome.runtime.onMessage.addListener(function(message, sender, sendResponse) {
        if (message.action === "updateUI") {
            chrome.storage.local.get(['paid', 'lifetimePaid'], function(result) {
                const isPaid = result.lifetimePaid || result.paid;
                console.log("Updated payment status from message: ", isPaid);
                updateUI(isPaid); // Use isPaid instead of true
            });
        }
    });

    // Check for stored lifetime plan status and update UI accordingly
    chrome.storage.local.get(['paid', 'lifetimePaid'], function(result) {
        const isPaid = result.lifetimePaid || result.paid;
        console.log("Initial payment status: ", isPaid);
        updateUI(isPaid);
    });

    // Get user data from ExtensionPay and update UI (only if lifetime plan isn't active)
    chrome.storage.local.get(['lifetimePaid'], function (result) {
        if (!result.lifetimePaid) {
            // User doesn't have lifetime plan, check with ExtensionPay
            extpay.getUser().then(user => {
                chrome.storage.local.set({ paid: user.paid });
                updateUI(user.paid);
            }).catch(err => {
                console.log("Error fetching data from ExtensionPay.");
            });

            // Listener for when user pays through ExtensionPay
            extpay.onPaid.addListener(user => {
                console.log('User paid with ExtensionPay!');
                chrome.storage.local.set({ paid: true });
                updateUI(true);
            });
        } else {
            // User has lifetime plan, set paid to true and update UI
            chrome.storage.local.set({ paid: true }, () => {
                updateUI(true);
            });
        }
    });

    // Function to update the UI based on payment status
    function updateUI(isPaid) {
        const manageButton = document.getElementById("manageButton");
        const manageButtonLifetime = document.getElementById("manageButtonLifetime");
        const upgradeButtonTop = document.getElementById("upgradeButtonTop");
        const settingsIcon = document.getElementById("settings_icon");
    
        if (isPaid) {
            // Check if lifetime or monthly
            chrome.storage.local.get(['lifetimePaid'], function(result) {
                if (result.lifetimePaid) {
                    manageButton.style.display = "none";
                    manageButtonLifetime.style.display = "block";
                    upgradeButtonTop.style.display = "block";
                } else {
                    manageButton.style.display = "block";
                    manageButtonLifetime.style.display = "none";
                    upgradeButtonTop.style.display = "block";
                }
            });
        } else {
            manageButton.style.display = "none";
            manageButtonLifetime.style.display = "none";
            upgradeButtonTop.style.display = "none";
        }
    
        document.getElementById("paymentButton").style.display = isPaid ? "none" : "block";
        document.getElementById("paymentButtonLifetime").style.display = isPaid ? "none" : "block";
        document.getElementById("paymentButtonPlace").style.display = isPaid ? "none" : "block";
        document.getElementById("loginButton").style.display = isPaid ? "none" : "block";
        document.getElementById("paidStatus").innerHTML = isPaid ? "<span style='color:#52d11c;'>Paid!</span>" : "Unpaid";
        document.getElementById("preDesc").innerHTML = isPaid ? "Pre-Activated Features:" : "Paid Pre-Activated Features:";
        document.getElementById("status").classList.toggle("unpaid", !isPaid);
        document.querySelectorAll("#statusReader").forEach(element => element.innerHTML = isPaid ? "ACTIVATED" : "OFF");
        settingsIcon.style.display = isPaid ? "flex" : "none";
    
        const toggleButton = document.getElementById("toggleButton");
        toggleButton.classList.toggle("unpaid", !isPaid);
        toggleButton.classList.toggle("paid", isPaid);
    }
    
    document.getElementById('paymentButtonPlace').addEventListener('click', function () {
        showOnlyPage(pageThree);
    });

    document.getElementById('pricing_back').addEventListener('click', function () {
        showOnlyPage(pageOne);
    }); 

    document.getElementById("manageButtonLifetime").addEventListener('click', function () {
        showOnlyPage(pageFour);
        chrome.storage.local.get(['customerEmail'], function (result) {
            if (result.customerEmail) {
                document.getElementById('customerEmailDisplay').textContent = result.customerEmail;
            }
        });
    });

    document.getElementById('manage_back').addEventListener('click', function () {
        showOnlyPage(pageOne);
    });  
    
    // Settings navigation
    const settingsIcon = document.getElementById("settings_icon");
    const updatesIcon = document.getElementById("updates_icon");
    
    const settingsBackIcon = document.getElementById("settings_back");
    const pageOne = document.getElementById("p1");
    const pageTwo = document.getElementById("p2");
    const pageThree = document.getElementById("p3");
    const pageFour = document.getElementById("p4");
    const pageFive = document.getElementById("p5");
    const pageSix = document.getElementById("p6");
    const pageSeven = document.getElementById("p7");

    settingsIcon.addEventListener("click", () => {
        settingsIcon.classList.toggle("activated");
        updatesIcon.classList.remove("activated"); // Only one active at a time
    });

    updatesIcon.addEventListener("click", () => {
        updatesIcon.classList.toggle("activated");
        settingsIcon.classList.remove("activated"); // Only one active at a time
    });

    settingsIcon.addEventListener('click', function () {
        settingsIcon.classList.toggle('activated');
        if (pageTwo.style.display === 'flex') {
            showOnlyPage(pageOne)
        } else {
            showOnlyPage(pageTwo);
        }
    });

    settingsBackIcon.addEventListener('click', function () {
        showOnlyPage(pageOne);
        settingsIcon.classList.remove('activated');
    });

    updatesIcon.addEventListener('click', function () {
        updatesIcon.classList.toggle('activated');
        settingsIcon.classList.remove("activated");
        if (pageSeven.style.display === 'flex') {
            showOnlyPage(pageOne);
        } else {
            showOnlyPage(pageSeven);
        }
    });


    // Toggle button handler
    document.getElementById('toggleButton').addEventListener('click', function () {
        chrome.tabs.query({ active: true, currentWindow: true }, function (tabs) {
            chrome.tabs.sendMessage(tabs[0].id, { action: "toggleBar" });
        });
    });

    const urlInput = document.getElementById('urlInput');
    const urlContainer = document.getElementById('urlContainer');
    const saveUrlsButton = document.getElementById('saveUrls');

    // Load saved URLs and update UI
    chrome.storage.local.get(['blockedUrls'], function (result) {
        if (result.blockedUrls) {
            result.blockedUrls.forEach(addUrlToUI);
        }
    });

    // Save URLs to storage without duplicates
    function saveUrls() {
        const urls = Array.from(new Set(Array.from(urlContainer.children).map(div => div.textContent.trim().replace('×', ''))));
        chrome.storage.local.set({ blockedUrls: urls }, function () {
            console.log('Blocked URLs saved.');
            // Re-run activate to refresh content script registration
            chrome.runtime.sendMessage({ action: 'updateBlockedUrls' });
        });
    }    

    // Save URLs when the Save button is clicked
    saveUrlsButton.addEventListener('click', function () {
        addUrlsFromInput();
        saveUrls();
    });

    // Add URL from input and avoid duplicates
    function addUrlsFromInput() {
        const inputUrls = urlInput.value.split(',').map(url => url.trim());
        const existingUrls = Array.from(urlContainer.children).map(div => div.textContent.trim().replace('×', ''));
        const newUrls = inputUrls.filter(url => url.length > 0 && !existingUrls.includes(url));
        newUrls.forEach(addUrlToUI);
        urlInput.value = '';  // Clear input after adding URLs
        saveUrls();  // Save immediately after adding
    }

    // Function to add a URL to the UI
    function addUrlToUI(url) {
        if (!url) return;
        const urlContainerDiv = document.createElement('div');
        urlContainerDiv.className = 'url-container';

        const urlDiv = document.createElement('div');
        urlDiv.className = 'url-item';
        urlDiv.textContent = url;

        const removeButton = document.createElement('i');
        removeButton.className = 'fa-solid fa-xmark-large remove-url';
        removeButton.onclick = function () {
            urlContainerDiv.remove();
            saveUrls();
        };

        urlDiv.appendChild(removeButton);
        urlContainerDiv.appendChild(urlDiv);
        urlContainer.appendChild(urlContainerDiv);
    }

    // Add event listener for 'Enter' and 'Comma' key presses
    urlInput.addEventListener('keydown', function (event) {
        if (event.key === 'Enter' || event.key === ',') {
            addUrlsFromInput();
            event.preventDefault();  // Prevent default action for Enter key
        }
    });

    document.getElementById('close-reopen').addEventListener('click', function() {
        window.close();
    
        chrome.runtime.reload();
    });

    document.getElementById('upgrade_back').addEventListener('click', function () {
        showOnlyPage(pageOne);
    });

    document.getElementById('updates_back').addEventListener('click', function () {
        showOnlyPage(pageOne);
        updatesIcon.classList.remove('activated');
    });

    document.getElementById('upgradeButton').addEventListener('click', function () {
        showOnlyPage(pageSix);
        settingsIcon.classList.add('activated');
        updatesIcon.classList.remove("activated");
    });

    document.getElementById('upgradeButtonContainer').addEventListener('click', function () {
        showOnlyPage(pageSix);
    });

    document.getElementById('paymentButtonLifetimeUpgrade').addEventListener('click', async function() {
        try {
            console.log("Creating Stripe session...");
            const response = await fetch(`${apiURL}/create-checkout-session`, {
                method: 'POST',
                headers: { 'Content-Type': 'application/json' },
            });
    
            if (response.ok) {
                const { url, session_id } = await response.json();
                console.log(`Stripe session created: ${session_id}`);
                window.open(url, 'Purchase', 'width=600,height=800,focused:true,left=450');
            } else {
                alert('Failed to create Stripe session.');
            }
        } catch (error) {
            console.error('Error creating Stripe session:', error);
        };

        document.getElementById('p6').style.display = 'none';
        document.getElementById('p5').style.display = 'flex';
    });    

    document.getElementById('manageButtonUpgrade').addEventListener('click', function() {
        extpay.openPaymentPage();
    });

    // URL of the GitHub file with pricing data (replace with the actual URL)
    const pricingURL = 'https://data.canvashack.com/prices.json';

    // Fetch the pricing data from the GitHub file
    async function fetchPricing() {
        try {
            const response = await fetch(pricingURL);
            if (response.ok) {
                const pricingData = await response.json();
                updatePricingUI(pricingData);
            } else {
                console.error('Failed to fetch pricing data');
            }
        } catch (error) {
            console.error('Error fetching pricing:', error);
        }
    }

    // Function to update the pricing on the UI
    function updatePricingUI(pricingData) {
        // Assuming pricingData has fields: monthly and oneTime
        const monthlyOriginal = pricingData.monthly[0];
        const monthlyDiscounted = pricingData.monthly[1];
        const oneTimeOriginal = pricingData.oneTime[0];
        const oneTimeDiscounted = pricingData.oneTime[1];

        // Update the monthly and one-time pricing elements
        document.querySelector('.monthly .original-price').textContent = `$${monthlyOriginal}`;
        document.querySelector('.monthly .discounted-price').textContent = `$${monthlyDiscounted}`;
        document.querySelector('.lifetime .original-price').textContent = `$${oneTimeOriginal}`;
        document.querySelector('.lifetime .discounted-price').textContent = `$${oneTimeDiscounted}`;
    }

    // Call fetchPricing to load the data on page load
    fetchPricing();

    const saveAnswersCheckbox = document.getElementById('saveAnswersCheckbox');

    // Load the saved state from chrome.storage.local
    chrome.storage.local.get(['saveCorrectAnswers'], function(result) {
        // Set the checkbox state based on the stored value, default to true
        saveAnswersCheckbox.checked = result.saveCorrectAnswers !== false;
    });

    // Add event listener to save the state when the checkbox is changed
    saveAnswersCheckbox.addEventListener('change', function() {
        const isChecked = saveAnswersCheckbox.checked;
        // Save the state to chrome.storage.local
        chrome.storage.local.set({ saveCorrectAnswers: isChecked }, function() {
            console.log('Save Correct Answers setting updated:', isChecked);
        });
    });

    chrome.storage.local.get('showPurchasePage', function(result) {
        if (result.showPurchasePage) {
            showOnlyPage(pageThree);

            chrome.storage.local.set({ showPurchasePage: false });
        }
    });

    const updatesDiv = document.getElementById('updates_content'); // Select the updates content div
    const importantAlertIcon = document.getElementById("important_alert_icon");

    fetch('https://data.canvashack.com/updates.json')
        .then(response => response.json())
        .then(data => {
            let hasImportant = false;
            let latestUpdateDate = null;

            // Function to calculate days ago from a given date
            function calculateDaysAgo(dateString) {
                const updateDate = new Date(dateString);
                const currentDate = new Date();
                const timeDifference = currentDate - updateDate;
                return Math.floor(timeDifference / (1000 * 60 * 60 * 24));
            }

            // Build content
            let content = '<div class="announcements-block">';
            data.updates.forEach((update, index) => {
                const daysAgo = calculateDaysAgo(update.date);
                content += `
                    <div class="announcements-box">
                        <h3>${update.date} <span class="days-ago">${daysAgo}d ago</span></h3>
                        <p>${update.title}</p>
                        <ul>
                            ${update.details.map(detail => `<li>${detail}</li>`).join('')}
                        </ul>
                        <p class="author">Blazer @ CanvasHack</p>
                    </div>
                `;
                
                // Check for the latest important update
                if (update.important && index === 0) {
                    hasImportant = true;
                    latestUpdateDate = update.date;
                }
            });
            content += '</div>';

            // Inject the content into the updates container
            updatesDiv.innerHTML = content;

            // Show exclamation icon if there’s a new important update
            chrome.storage.local.get(['lastViewedUpdateDate'], function(result) {
                if (hasImportant && result.lastViewedUpdateDate !== latestUpdateDate) {
                    importantAlertIcon.style.display = 'block';
                }
            });

            // Event listener for updates icon to clear alert when viewed
            updatesIcon.addEventListener('click', () => {
                if (hasImportant) {
                    chrome.storage.local.set({ lastViewedUpdateDate: latestUpdateDate });
                    importantAlertIcon.style.display = 'none';
                }
            });
        })
        .catch(error => console.error('Error fetching updates:', error));

        let currentStep = 0;
        const steps = document.querySelectorAll(".step");
        const progressFill = document.querySelector(".progress-fill");
    
        // Initialize progress to start at 25%
        progressFill.style.width = "25%";
    
        // Display the first step
        steps[currentStep].classList.add("active");
    
        function updateStep() {
            steps.forEach((step, index) => step.classList.toggle("active", index === currentStep));
            
            // Calculate progress, starting from 25%
            const progressPercentage = 25 + ((currentStep / (steps.length - 1)) * 75);
            progressFill.style.width = `${progressPercentage}%`;
        }
    
        document.querySelectorAll(".next-btn").forEach((button) => {
            button.addEventListener("click", () => {
                if (currentStep < steps.length - 1) {
                    currentStep++;
                    updateStep();
                } else {
                    // Finish the tutorial
                    document.getElementById("step-container").style.display = "none";
                    localStorage.setItem("tutorialCompleted", "true");
                }
            });
        });
    
        // Load tutorial only if not previously completed
        if (!localStorage.getItem("tutorialCompleted")) {
            document.getElementById("step-container").style.display = "flex";
        } else {
            document.getElementById("step-container").style.display = "none";
        }

        var downArrow = document.getElementById('down-arrow');
        var reviewColumn = document.querySelector('.review-column');
        var scrollInterval; // Variable to store the interval ID
      
        // Function to start scrolling
        function startScrolling() {
          clearInterval(scrollInterval); // Clear any existing intervals
          scrollInterval = setInterval(function() {
            reviewColumn.scrollBy({
              top: 10, // Adjust scroll amount
              left: 0,
              behavior: 'smooth'
            });
          }, 8); // Adjust interval duration
        }
      
        // Function to stop scrolling
        function stopScrolling() {
          clearInterval(scrollInterval);
        }

        downArrow.addEventListener('click', function() {
            // Scroll the review column down by 50 pixels
            reviewColumn.scrollBy({
              top: 140, // Scroll down 50 pixels
              left: 0,
              behavior: 'smooth' // Optional: Smooth scrolling effect
            });
          });
      
        // Event listeners for mouse events
        downArrow.addEventListener('mousedown', startScrolling);
        downArrow.addEventListener('mouseup', stopScrolling);
        downArrow.addEventListener('mouseleave', stopScrolling);
      
        // Event listeners for touch events (mobile devices)
        downArrow.addEventListener('touchstart', function(e) {
          e.preventDefault();
          startScrolling();
        });
        downArrow.addEventListener('touchend', stopScrolling);

        const manageButtons = document.querySelectorAll('#manageButton, #manageButtonLifetime, #manageButtonUpgrade');
    const upgradeButtonSwitch = document.getElementById('upgradeButtonSwitch');
    const upgradeButtons = document.querySelectorAll('#upgradeButtonTop, #upgradeButtonContainer, #upgradeButton');

    // Load saved state for Upgrade Button Switch
    const upgradeButtonState = localStorage.getItem('upgradeButtonHidden') === 'true';
    upgradeButtonSwitch.checked = !upgradeButtonState; // Checked means not hidden

    // Apply initial display state and manage button text based on saved preference
    upgradeButtons.forEach(button => button.style.display = upgradeButtonState ? 'none' : 'block');
    manageButtons.forEach(button => button.textContent = upgradeButtonState ? "Manage Account" : "Manage");

    upgradeButtonSwitch.addEventListener('change', function () {
        const hideUpgradeButtons = !upgradeButtonSwitch.checked;
        
        upgradeButtons.forEach(button => button.style.display = hideUpgradeButtons ? 'none' : 'block');
        manageButtons.forEach(button => button.textContent = hideUpgradeButtons ? "Manage Account" : "Manage");
        
        // Save the state in localStorage
        localStorage.setItem('upgradeButtonHidden', hideUpgradeButtons);
    });
});
